# Demo for open-source weather API 'Open-Meteo'
# https://open-meteo.com/
# Version 2
# by 'jaydeel' at ipcamtalk dot com
# Inspired by @Chris Moore:  https://ipcamtalk.com/threads/api-connection-down.67974/post-743731
# Update history:
#	v2 - 2023-03-27 - Updated $conditions variable
#   v1 - 2023-02-17 - First release

# ================================================
# USER SETTINGS...

# YOUR GEO LOCATION
$lat = 37.00000
$lon = -122.00000
$tz = "America/Los_Angeles"

# [OPTIONAL] SAVE RESULTS TO *.TXT FILES FOR VIDEO OVERLAYS
$path = 'C:\Program Files (x86)\Blue Iris Tools\txt'
#$path = 'C:\Blue Iris\My Overlays\openmeteo'

# ================================================

# QUERY THE OPEN-METEO API
# ... THE API RETURNS JSON OBJECT LITERALS WITH KEY/VALUE PAIRS LIKE...
#    "current_weather":{"temperature":7.1,"windspeed":7.4,"winddirection":227.0,"weathercode":3,"time":"2023-02-16T23:00"}
#    "hourly:{"precipitation":[0.000, ... ,0.004]}  (1 week of hourly values, 168 total, oldest to newest)
#    "daily:{"sunrise":["2023-02-17T06:53", ... ,"2023-02-23T06:46"]}  (7 values, today first)
# 
$uri = "https://api.open-meteo.com/v1/forecast?latitude=$lat&longitude=$lon&hourly=temperature_2m,relativehumidity_2m,dewpoint_2m,precipitation,cloudcover&daily=sunrise,sunset,precipitation_sum&current_weather=true&temperature_unit=fahrenheit&windspeed_unit=mph&precipitation_unit=inch&timezone=$tz"
$webData = Invoke-RestMethod -URI $uri

# EXTRACT WEATHER VALUES
$time         = $webData.current_weather.time
$code         = $webData.current_weather.weathercode
$temp         = $webData.current_weather.temperature
$wspd         = $webData.current_weather.windspeed
$wdir         = $webData.current_weather.winddirection  # 0-360 degrees
$time_ary     = $webData.hourly.time
$temp_2m_ary  = $webData.hourly.temperature_2m
$rh_2m_ary    = $webData.hourly.relativehumidity_2m
$dewpt_2m_ary = $webData.hourly.dewpoint_2m
$ppt_ary      = $webData.hourly.precipitation
$cldcvr_ary   = $webData.hourly.cloudcover
$sunrise_ary  = $webData.daily.sunrise
$sunset_ary   = $webData.daily.sunset
$ppt_sum_ary  = $webData.daily.precipitation_sum

# GET CONDITION FROM CODE#
<# 0	Clear sky
1, 2, 3	Mainly clear, partly cloudy, and overcast
45, 48	Fog and depositing rime fog
51, 53, 55	Drizzle: Light, moderate, and dense intensity
56, 57	Freezing Drizzle: Light and dense intensity
61, 63, 65	Rain: Slight, moderate and heavy intensity
66, 67	Freezing Rain: Light and heavy intensity
71, 73, 75	Snow fall: Slight, moderate, and heavy intensity
77	Snow grains
80, 81, 82	Rain showers: Slight, moderate, and violent
85, 86	Snow showers slight and heavy
95 *	Thunderstorm: Slight or moderate
96, 99 *	Thunderstorm with slight and heavy hail #>
$conditions = [ordered]@{}
$conditions['0'] = 'Clear sky'
$conditions['1'] = 'Mainly clear'
$conditions['2'] = 'Partly cloudy'
$conditions['3'] = 'Overcast'
$conditions['45'] = 'Fog'
$conditions['48'] = 'Depositing rime fog'
$conditions['51'] = 'Drizzle: Light intensity'
$conditions['53'] = 'Drizzle: Moderate intensity'
$conditions['55'] = 'Drizzle: Dense intensity'
$conditions['56'] = 'Freezing Drizzle: Light'
$conditions['57'] = 'Freezing Drizzle: Dense intensity'
$conditions['61'] = 'Rain: Slight intensity'
$conditions['63'] = 'Rain: Moderate intensity'
$conditions['65'] = 'Rain: Heavy intensity'
$conditions['66'] = 'Freezing Rain: Light intensity'
$conditions['67'] = 'Freezing Rain: Heavy intensity'
$conditions['71'] = 'Snow fall: Slight intensity'
$conditions['73'] = 'Snow fall: Moderate intensity'
$conditions['75'] = 'Snow fall: Heavy intensity'
$conditions['77'] = 'Snow grains'
$conditions['80'] = 'Rain showers: Slight'
$conditions['81'] = 'Rain showers: Moderate'
$conditions['82'] = 'Rain showers: Violent'
$conditions['85'] = 'Snow showers slight'
$conditions['86'] = 'Snow showers heavy'
$conditions['95'] = 'Thunderstorm: Slight'
$conditions['96'] = 'Thunderstorm with slight hail'
$conditions['99'] = 'Thunderstorm with heavy hail'
$condition = $conditions[$code]

# GET WIND COMPASS DIRECTION 
$wind_sector = $webData.current_weather.winddirection/22.5 # Divide the angle by 360/16 directions = 22.5 deg/direction
$wind_sector = [math]::Round($wind_sector, 0)
$WindCompassDirection = @("N","NNE","NE","ENE","E","ESE", "SE", "SSE","S","SSW","SW","WSW","W","WNW","NW","NNW","N")
$wdir_compass = $WindCompassDirection[$wind_sector]

# GET CURRENT VALUES FROM HOURLY ARRAYS
$cur_index    = $time_ary.indexof($time)
$cur_temp_2m  = $temp_2m_ary[$cur_index]
$cur_rh_2m    = $rh_2m_ary[$cur_index]
$cur_dewpt_2m = $dewpt_2m_ary[$cur_index]
$cur_ppt      = $ppt_ary[$cur_index]
$cur_clouds   = $cldcvr_ary[$cur_index]
$cur_sunrise  = $sunrise_ary[0]
$cur_sunset   = $sunset_ary[0]
$cur_pptsum   = $ppt_sum_ary[0]

# SHOW RESULTS
Write-Host "======================================================="
Write-Host "Demo for open-source weather API 'Open-Meteo'"
Write-Host "(free, no-registration, up to 10,000 calls per day)"
Write-Host "-------------------------------------------------------"
Write-Host "Time                = $time"
Write-Host "Condition           = $condition [code=$code]"
Write-Host "Temperature (F)     = $temp"
Write-Host "Wind speed (mph)    = $wspd"
Write-Host "Wind direction      = $wdir_compass ($wdir degrees)"
Write-Host "Precipitation (in)  = $cur_ppt (hourly total)"
Write-Host "Temperature @2m (F) = $cur_temp_2m"
Write-Host "RH @2m (%)          = $cur_rh_2m"
Write-Host "Dewpoint @2m (F)    = $cur_dewpt_2m"
Write-Host "Cloudcover (%)      = $cur_clouds"
Write-Host "Sunrise             = $cur_sunrise"
Write-Host "Sunset              = $cur_sunset"
Write-Host "Precipitation (in)  = $cur_pptsum (daily total)"

# SAVE *.TXT FILES - LIKE THOSE CREATED BY BLUE IRIS TOOLS (BIT)
if ($path) {
	echo $time | Out-File -FilePath "$path\timeupdated.txt" -encoding default -nonewline
	echo $temp '°F' | Out-File -FilePath "$path\temperature.txt" -encoding default -nonewline
	echo $condition | Out-File -FilePath "$path\condition.txt" -encoding default -nonewline
	echo $wspd ' mph' | Out-File -FilePath "$path\windspeed.txt" -encoding default -nonewline
	echo $wdir_compass " (" $wdir "°)" | Out-File -FilePath "$path\winddescription.txt" -encoding default -nonewline
	echo $cur_ppt ' in' | Out-File -FilePath "$path\precipitation_hourly.txt" -encoding default -nonewline
	echo $cur_rh_2m '%' | Out-File -FilePath "$path\humidity.txt" -encoding default -nonewline
	echo $cur_dewpt_2m '°F' | Out-File -FilePath "$path\dewpoint.txt" -encoding default -nonewline
	echo $cur_clouds '%' | Out-File -FilePath "$path\cloudcover.txt" -encoding default -nonewline
	echo $cur_sunrise | Out-File -FilePath "$path\sunrise.txt" -encoding default -nonewline
	echo $cur_sunset | Out-File -FilePath "$path\sunset.txt" -encoding default -nonewline
	echo $cur_pptsum ' in' | Out-File -FilePath "$path\precipitation_daily.txt" -encoding default -nonewline

	Write-Host "`nNote: values saved as *.txt files in '$path'"
}

# SHOW HELPFUL URLS
Write-Host "`nURL: to view full results of the API call...`n$uri"
Write-Host "`nURLs: to learn more..."
Write-Host "https://open-meteo.com/"
Write-Host "https://open-meteo.com/en/docs"
Write-Host "https://github.com/open-meteo/open-meteo"

Write-Host ""
